"""
The aisquared.config.rendering subpackage contains objects for configuring how rendering of predictions is to occur.
"""

from .ImageRendering import ImageRendering
from .ObjectRendering import ObjectRendering
from .DocumentRendering import DocumentRendering
from .WordRendering import WordRendering
from .FilterRendering import FilterRendering
