"""
The aisquared.config.harvesting subpackage contains objects for configuring harvesting of data.
"""

from .ImageHarvester import ImageHarvester
from .TextHarvester import TextHarvester
from .InputHarvester import InputHarvester
