"""
The aisquared.serving package contains utilities to serve models to a local REST endpoint
"""

from mlflow.tensorflow import save_model as save_tensorflow_model
from mlflow.sklearn import save_model as save_sklearn_model
from mlflow.pytorch import save_model as save_pytorch_model
from mlflow.keras import save_model as save_keras_model

from .deploy_model import deploy_model, load_mann_model, load_keras_model, load_pytorch_model, load_sklearn_model, load_tensorflow_model
from .get_remote_prediction import get_remote_prediction
