"""
The aisquared.config subpackage contains utilities and objects for packaging aisquared configuration steps and models.
"""

from .ModelConfiguration import ModelConfiguration
from .GraphConfiguration import GraphConfiguration
